/**
 * Temporal Quantified fuzzy DL's
 */
package monitorDb;

import java.sql.DriverManager;
import java.sql.SQLException;

import reasoner.Concept;
import reasoner.Individual;
import reasoner.EventInd;
import reasoner.KnowledgeBase;
import reasoner.Role;
import reasoner.ConcreteConcept;
import reasoner.ULineEvent;

/**
 * The class for accessing a DBMS and converting data in ABox
 * for a given KnowledgeBase
 * 
 * @version 1.0 10/12/07
 * @author marco furlan
 *
 */
public class DatabaseToKb extends Database {	
	private KnowledgeBase kb;
	
	
	/**
	 * Generates a DatabaseToKb instance and connects it to a DBMS via jdbc. 
	 * The String propFile is  property file name with connection driver name 
	 * and db simbolic name.
     * @param propFile the property file url
     * @throws Exception
     */	 
	public DatabaseToKb(String propFile) throws Exception {
		super(propFile);		
	}

	/**
	 * Generates a DatabaseToKb instance and connects it to a DBMS via jdbc, using a default 
     * property file with connection driver name and db simbolic name.
     * @throws Exception
	 */
	public DatabaseToKb() throws Exception {
		super();
	}

	
	/**
	 * Generates a DatabaseToKb instance and connects it to a DBMS via jdbc. 
	 * The String propFile is  property file name with connection driver name 
	 * and db simbolic name. Receives a KnowledgeBase
     * @param propFile the property file url
	 * @param kb the KnowledgeBase
	 * @throws Exception
	 */
	public DatabaseToKb(String propFile, KnowledgeBase kb) throws Exception {
		this(propFile);		
		this.kb = kb;
	}
	
    /**
     * Executes a Select sql query to the database and convert the obtained table into ABox, creating
     * a set of new ULineEvent and inserting them into KB 
     * @param query the sql query
     * @param r a Role in the KB
     * @param tc a ConcreteConcept
     * @throws SQLException
     */
    public synchronized void setABox (String query, Role r, ConcreteConcept tc) throws SQLException {
        con = DriverManager.getConnection (url, user, "");
        stm = con.createStatement();
        ULineEvent ul;    
        Individual ind;        
        Concept c = tc.toConcept();
        rs = stm.executeQuery (query);
        rsmt = rs.getMetaData();
        int l = rsmt.getColumnCount();
        String name,s = new String();
        double d, val;
        while (rs.next()) {
        	name = rs.getString(rsmt.getColumnName(1));
        	ul = new ULineEvent(name, getKb());
        	name = name.toLowerCase();
        for (int k = 2; k <= l; k++) {
            s = rsmt.getColumnName(k);                            
            d = rs.getDouble(s);
            ind = new EventInd(name+s,ul,Double.parseDouble(s), getKb());
            r.add(ind, 1.0);
            val = tc.qvalue(d);
            c.add(ind, val);
            }            
        }
        rs.close();
        stm.close();
        con.close();        
    }
	
    /**
     * Executes a Select sql query to the database and convert the obtained table into ABox, creating
     * a set of new ULineEvent and inserting them into KB 
     * @param query the sql query
     * @param tc1 first ConcreteConcept
     * @param tc2 second ConcreteConcept
     * @throws SQLException
     */
    public synchronized void setABox (String query, ConcreteConcept tc1, ConcreteConcept tc2) throws SQLException {
        con = DriverManager.getConnection (url, user, "");
        stm = con.createStatement();
        ULineEvent ul;    
        Individual ind;        
        Concept c1,c2;
        c1= tc1.toConcept();
        c2= tc2.toConcept();
        rs = stm.executeQuery (query);
        rsmt = rs.getMetaData();
        int l = rsmt.getColumnCount();
        String name,s = new String();
        double d, val1, val2;
        while (rs.next()) {
        	name = rs.getString(rsmt.getColumnName(1));
        	ul = new ULineEvent(name, getKb());
        	name = name.toLowerCase();
        for (int k = 2; k <= l; k++) {
            s = rsmt.getColumnName(k);                            
            d = rs.getDouble(s);
            ind = new EventInd(name+s,ul,Double.parseDouble(s), getKb());
            
            val1 = tc1.qvalue(d);
            c1.add(ind, val1);
            val2 = tc2.qvalue(d);
            c2.add(ind, val2);
            }            
        }
        rs.close();
        stm.close();
        con.close();        
    }

	/**
	 * @return the kb
	 */
	public KnowledgeBase getKb() {
		return kb;
	}

	/**
	 * @param kb the kb to set
	 */
	public void setKb(KnowledgeBase kb) {
		this.kb = kb;
	}
}
