/**
 * Temporal Quantified fuzzy DL's
 */
package monitorDb;

import java.awt.Dimension;
import java.awt.Toolkit;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.text.NumberFormat;
import java.util.List;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;
import reasoner.*;
import tbox.QuantifierType;
import tbox.QueryType;
import tbox.RoleType;
import tbox.SimpleConcept;
import tbox.TBoxType;
import tbox.TrapeziumType;



/**
 * A monitor for evaluation, able to load and convert an xml file into a KnowledgeBase containing MTBox
 * and a Query, to load data from a database, to convert data into ABox and to evaluate the Query.
 * 
 * @version 1.0 10/12/07
 * @author marco furlan
 *
 */
public class Monitor {
	private static MonitorJFrame monitorFrame;
	private DatabaseToKb db;
	private String propFile = "monitorDb.properties";	
	private String tableName = "CityTemperature";
	private static TBoxType mtbox;
	private static ConcreteConcept tc, tc2;
	private static Concept c, c2;
	private static Quantifier tq,q;
	private static Role r;
	private static Query qry;
	private QueryType qryT;
	private static boolean ulCreated;
	private String evalStr;
	private GraphicULineSet graphULs;
	private static NumberFormat formatter;
	private boolean subs;

	private KnowledgeBase kb;

	/**
	 * Constructor
	 */
	public Monitor(){
		monitorFrame = new MonitorJFrame(this);
		Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
		monitorFrame.setSize(d.width*5/6, d.height*5/6);
		monitorFrame.setLocation((d.width - monitorFrame.getSize().width) / 8,
				(d.height - monitorFrame.getSize().height) / 8);
		monitorFrame.setVisible(true);
		formatter = NumberFormat.getNumberInstance();
		formatter.setMaximumFractionDigits(6);	
		setSubs(false);
	}
	/**
	 * Constructor
	 */
	public Monitor(String tableName){
		this();
		this.monitorFrame.getTitleLbl().setText(tableName + " MONITOR");
		this.setTableName(tableName);
	}
	/**
	 * Constructs a new Monitor. First argument, if exists, gives property file name
	 * @param args the arguments
	 */
	public static void main(String[] args) {
		if (null!=args && args.length>0){			
		Monitor monitor = new Monitor(args[0]);
		if (args.length>1)
			monitor.setPropFile(args[1]);	
		}      
	}
	/**
	 * prints the MTBox in String format
	 */
	public void printMTBox() {
		String constr = getKb().toString();
		setMTBoxText(constr);		
	}
	/**
	 * @param s the String to set
	 */
	private void setMTBoxText(String s) {
		monitorFrame.setText1(s);		
	}

	/**
	 * Connects to database referred by property file and create the relative ABox
	 * @throws Exception
	 */
	public void setABox() throws Exception{
		db = new DatabaseToKb(propFile, getKb());
		String sql =
			"SELECT * FROM " + getTableName();
		if (qryT.isIsRoleQuery()&& r!=null && tc!=null)
			db.setABox(sql, r, tc);	
		if (!qryT.isIsRoleQuery()&& tc!=null && tc2!=null)
			db.setABox(sql, tc, tc2);		
		graphULs = new GraphicULineSet();
//		graphULs.create(c, getKb());
		graphULs.create(qry, getKb());
	}
	/**
	 * Loads MTBox from an xml file
	 * @param source the xml file
	 * @throws JAXBException
	 * @throws FileNotFoundException
	 */
	public  void loadMTBox(File source) throws JAXBException, FileNotFoundException {
		ulCreated = false;
		setSubs(false);
		JAXBContext ctx = JAXBContext.newInstance("tbox");
		Unmarshaller um = ctx.createUnmarshaller();
		JAXBElement<TBoxType> mboxXML = (JAXBElement<TBoxType>)um.unmarshal(new FileInputStream(source));		
		mtbox = mboxXML.getValue();	
		setKb(new KnowledgeBase());
		List<SimpleConcept> scList = (List<SimpleConcept>)mtbox.getSimpleConcepts();
		for(SimpleConcept sc : scList){
			TrapeziumType t = sc.getTrapezium();
			tc = new ConcreteConcept(t.getName(),t.getXA(),t.getXB(),t.getXC(),
					t.getXD(),t.getMaxVal(),getKb());
		}
		List<QuantifierType> qList = (List<QuantifierType>)mtbox.getQuantifiers();
		for(QuantifierType qt : qList){
			TrapeziumType t = qt.getTrapezium();
			q = new Quantifier(t.getName(),t.getXA(),t.getXB(),t.getXC(),
					t.getXD(),t.getMaxVal(),getKb());
			q.setRelative(qt.isRelative());
		}
		List<RoleType> rList = (List<RoleType>)mtbox.getRoles();
		for(RoleType rt : rList){
			String name = rt.getName();
			if(name.equals("distance"))
				r = new Distance(rt.getName(),getKb());
			else
				r = new Role(rt.getName(),getKb());			
		}
		/////LAST!!!!
		qryT = mtbox.getQuery();
		if (qryT!=null){
			String tqname, qname, rname, op, tcname, tcname2;
			if (qryT.isIsRoleQuery()){	// qry is RoleQuery
				tqname = qryT.getTempQuantifier();			
				qname = qryT.getQuantifier();
				rname = qryT.getRole();
				tcname = qryT.getConcept();
				if(null!=tqname && getKb().getQuantifiers().containsKey(tqname))
					tq = (Quantifier)getKb().getQuantifiers().get(tqname);
				else tq = null;
				if(null!=qname && getKb().getQuantifiers().containsKey(qname))
					q = (Quantifier)getKb().getQuantifiers().get(qname);
				else q = null;
				if(getKb().getRoles().containsKey(rname))
					r = (Role)getKb().getRoles().get(rname);
				if(getKb().getConcrConcepts().containsKey(tcname)){
					tc = (ConcreteConcept)getKb().getConcrConcepts().get(tcname);
					c = tc.toConcept();
				}
				qry = new RoleQuery(tq,q,r,tc,getKb());		//double quantification!!
			}else{    //qry is OpQuery
				qname = qryT.getQuantifier();
				tcname = qryT.getConcept();
				op = qryT.getOperator();
				tcname2 = qryT.getConcept2();
				if (op.trim().equalsIgnoreCase("subsumedNext")||
						op.trim().equalsIgnoreCase("SubsNext"))
					setSubs(true);					
				if(null!=qname && getKb().getQuantifiers().containsKey(qname))
					q = (Quantifier)getKb().getQuantifiers().get(qname);
				if(getKb().getConcrConcepts().containsKey(tcname)){
					tc = (ConcreteConcept)getKb().getConcrConcepts().get(tcname);
					c = tc.toConcept();
				}
				if(getKb().getConcrConcepts().containsKey(tcname2)){
					tc2 = (ConcreteConcept)getKb().getConcrConcepts().get(tcname2);
					c2 = tc2.toConcept();
				}
				if (null!=q)
					qry = new OpQuery(q,tc, op, tc2, getKb());
				else
					qry = new OpQuery(tc, op, tc2, getKb());
			}
		}
		if(null!=this.getGraphicULineSet())
			this.getGraphicULineSet().setCreated(false);
	}
	/**
	 * Graphically shows the ABox
	 */
	public void showABox(){	
		getMF().getGp().setShow(true);
		getMF().getGp().repaint();
		getMF().getLegenda().repaint();
	}
	/**
	 * Evaluates the Query in the current KnowledgeBase and show the result
	 */
	public void evaluate(boolean fuzzy){
		evalStr = "";
		String s = "";
		if (qry!=null){
			try{
				if (qryT.isIsRoleQuery()){
					s += ((RoleQuery)qry).evaluateString();
				}else{			//OpQuery
					 s += ((OpQuery)qry).evaluateString(false, fuzzy);//not incremental: evaluation from the beginning					
				}
			}catch(Exception ex){
				ex.printStackTrace();
			}
			evalStr += "Query evaluation: \n" + s;
		}
		printEvaluation();
	}
	/**
	 * Prints the evaluation text
	 */
	private void printEvaluation() {
		monitorFrame.setEvalText(evalStr);

	}
	/**
	 * Gets the KnowledgeBase
	 * @return the kb
	 */
	public KnowledgeBase getKb() {
		return kb;
	}
	/**
	 * Sets the KnowledgeBase
	 * @param kb the kb to set
	 */
	public void setKb(KnowledgeBase kb) {
		this.kb = kb;
	}

	/**
	 * Gets the Query
	 * @return qry the Query
	 */
	public static Query getQry() {
		return qry;
	}

	/**
	 * Sets the Query
	 * @param qry the Query to set
	 */
	public static void setQry(Query qry) {
		Monitor.qry = qry;
	}

	/**
	 * @return c the Concept
	 */
	public static Concept getC() {
		return c;
	}

	/**
	 * @param c the Concept to set
	 */
	public static void setC(Concept c) {
		Monitor.c = c;
	}

	/**
	 * @return the monitorJFrame
	 */
	public static MonitorJFrame getMF() {
		return monitorFrame;
	}

	/**
	 * @param monitorFrame the monitorJFrame to set
	 */
	public static void setMF(MonitorJFrame monitorFrame) {
		Monitor.monitorFrame = monitorFrame;
	}

	/**
	 * @return the graphULs
	 */
	public GraphicULineSet getGraphicULineSet() {
		return graphULs;
	}

	/**
	 * @param graphULs the graphULs to set
	 */
	public void setGraphicULineSet(GraphicULineSet graphULs) {
		this.graphULs = graphULs;
	}
	/**
	 * @return the tableName
	 */
	public String getTableName() {
		return tableName;
	}
	/**
	 * @param tableName the tableName to set
	 */
	public void setTableName(String tableName) {
		this.tableName = tableName;
	}
	/**
	 * @return the propFile
	 */
	public String getPropFile() {
		return propFile;
	}
	/**
	 * @param propFile the propFile to set
	 */
	public void setPropFile(String propFile) {
		this.propFile = propFile;
	}
	/**
	 * @return the subs
	 */
	public boolean isSubs() {
		return subs;
	}
	/**
	 * @param subs the subs to set
	 */
	public void setSubs(boolean subs) {
		this.subs = subs;
	}
}
