/**
 * Temporal Quantified fuzzy DL's
 */
package monitor;


import java.awt.Dimension;
import java.awt.Toolkit;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.text.NumberFormat;
import java.util.List;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;
import javax.xml.datatype.XMLGregorianCalendar;

import reasoner.*;

import tbox.*;


/**
 * A monitor for a pray-predator system, able to load a MTBox and a Query from an xml file, to load
 * a current state from an other xml file, to convert it into ABox and eventually to evaluate the Query.
 * It writes also a report in a log file.
 * 
 * @version 1.0 10/12/07
 * @author marco furlan
 *
 */
public class Monitor implements Runnable{
	private Thread thread;
	private static MonitorJFrame monitorFrame;
	private static String outStr= "";
	private static String outFile = "log.txt"; 
	private static String evalStr = "";    
	private File stateFile = null;
	private static OutputStreamWriter sw;
	private static CurrentStateType state;
	private static TBoxType mtbox;
	private static List<NodeType> agents;
	private static ConcreteConcept tc, tc2;
	private static Concept c,c2;
	private static Quantifier tq,q;
	private static Role r;
	private static NumberFormat formatter;
	private static KnowledgeBase kb;
	private static Query qry;
	private QueryType qryT;
	private static boolean ulCreated;
	private boolean subs;
	private boolean fuzzy;

	/**
	 * Constructor
	 */
	public Monitor() {
		super();	
		monitorFrame = new MonitorJFrame(this);
		Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
		monitorFrame.setSize(d.width*5/6, d.height*5/6);
		monitorFrame.setLocation((d.width - monitorFrame.getSize().width) / 8,
				(d.height - monitorFrame.getSize().height) / 8);
		monitorFrame.setVisible(true);
		formatter = NumberFormat.getNumberInstance();
		formatter.setMaximumFractionDigits(6);	
		this.setSubs(false);
		this.setFuzzy(true);
	}

	/**
	 * Prints the MTBox in String format
	 */
	public void printMTBox() {
		String constr = getKb().toString();
		setMTBoxText(constr);		
	}

	/**
	 * @param s the String to set
	 */
	private void setMTBoxText(String s) {
		monitorFrame.setText1(s);		
	}

	/**
	 * Loads the current state from an xml file
	 * @param file the xml file
	 * @throws JAXBException
	 * @throws FileNotFoundException
	 */
	public static void loadCurrState(File file) throws JAXBException, FileNotFoundException{
		JAXBContext ctx = JAXBContext.newInstance("monitor");
		Unmarshaller um = ctx.createUnmarshaller();
		JAXBElement<CurrentStateType> stateXML = (JAXBElement<CurrentStateType>)um.unmarshal(new FileInputStream(file));
		state = stateXML.getValue();		
		agents = (List<NodeType>)state.getNodes();                
	}

	/**
	 * Loads MTBox from an xml file
	 * @param source the xml file
	 * @throws JAXBException
	 * @throws FileNotFoundException
	 */
	public  void loadMTBox(File source) throws JAXBException, FileNotFoundException {
		ulCreated = false;
		this.setSubs(false);
		this.setFuzzy(true);
		JAXBContext ctx = JAXBContext.newInstance("tbox");
		Unmarshaller um = ctx.createUnmarshaller();
		JAXBElement<TBoxType> mboxXML = (JAXBElement<TBoxType>)um.unmarshal(new FileInputStream(source));		
		mtbox = mboxXML.getValue();	
		setKb(new KnowledgeBase());
		List<SimpleConcept> scList = (List<SimpleConcept>)mtbox.getSimpleConcepts();
		for(SimpleConcept sc : scList){
			TrapeziumType t = sc.getTrapezium();
			tc = new ConcreteConcept(t.getName(),t.getXA(),t.getXB(),t.getXC(),
					t.getXD(),t.getMaxVal(),getKb());
		}
		List<QuantifierType> qList = (List<QuantifierType>)mtbox.getQuantifiers();
		for(QuantifierType qt : qList){
			TrapeziumType t = qt.getTrapezium();
			q = new Quantifier(t.getName(),t.getXA(),t.getXB(),t.getXC(),
					t.getXD(),t.getMaxVal(),getKb());
			q.setRelative(qt.isRelative());
		}
		List<RoleType> rList = (List<RoleType>)mtbox.getRoles();
		for(RoleType rt : rList){
			String name = rt.getName();
			if(name.equals("distance"))
				r = new Distance(rt.getName(),getKb());
			else
				r = new Role(rt.getName(),getKb());			
		}
		/////LAST!!!!
		qryT = mtbox.getQuery();
		if (qryT!=null){
			String tqname, qname, rname, op, tcname, tcname2;
			if (qryT.isIsRoleQuery()){	// qry is RoleQuery
				tqname = qryT.getTempQuantifier();	
				qname = qryT.getQuantifier();
				rname = qryT.getRole();
				tcname = qryT.getConcept();
				if(null!=tqname && getKb().getQuantifiers().containsKey(tqname))
					tq = (Quantifier)getKb().getQuantifiers().get(tqname);	
				else tq = null;
				if(null!=qname && getKb().getQuantifiers().containsKey(qname))
					q = (Quantifier)getKb().getQuantifiers().get(qname);
				else q = null;
				if(getKb().getRoles().containsKey(rname))
					r = (Role)getKb().getRoles().get(rname);
				if(getKb().getConcrConcepts().containsKey(tcname)){
					tc = (ConcreteConcept)getKb().getConcrConcepts().get(tcname);
					c = tc.toConcept();
				}
				qry = new RoleQuery(tq,q,r,tc,getKb());		//double quantification!!
			}else{    //qry is OpQuery
				qname = qryT.getQuantifier();
				tcname = qryT.getConcept();
				op = qryT.getOperator();
				tcname2 = qryT.getConcept2();
				if (op.trim().equalsIgnoreCase("subsumedNext")||
						op.trim().equalsIgnoreCase("SubsNext"))
					setSubs(true);	
				if(null!=qname && getKb().getQuantifiers().containsKey(qname))
					q = (Quantifier)getKb().getQuantifiers().get(qname);
				if(getKb().getConcrConcepts().containsKey(tcname)){
					tc = (ConcreteConcept)getKb().getConcrConcepts().get(tcname);
					c = tc.toConcept();
				}
				if(getKb().getConcrConcepts().containsKey(tcname2)){
					tc2 = (ConcreteConcept)getKb().getConcrConcepts().get(tcname2);
					c2 = tc2.toConcept();
				}
				if (null!=q)
					qry = new OpQuery(q,tc, op, tc2, getKb());
				else
					qry = new OpQuery(tc, op, tc2, getKb());
			}
		}
	}
	/**
	 * By the current geometrical state, constructs the ABox and calls the reasoner
	 * for evaluating the query
	 */
	public void setState(){
		XMLGregorianCalendar time =state.getTime();
		long tsec =time.toGregorianCalendar().getTimeInMillis();
		outStr = "";
		outStr += " time: " + time + "\n";
		outStr += " agents number: " + (agents.size()-1) + "\n";
		evalStr = "";
		ULineEvent ul;
		Individual ind;
		double x,y,d,e,e2,qeval;
//		for (NodeType node : agents){
		for (int i =1; i < agents.size();i++){	//per ogni agente escluso pray
			NodeType node = agents.get(i);
			String name = node.getName();
			String nametime = name.toLowerCase() + tsec;
			ul = (ULineEvent)getKb().getULineEvents().get(name);
			ind = new EventInd(nametime,ul,tsec,getKb());			//nel caso di query con operatori temporali
//			ind = new Individual(name.toLowerCase(),ul,tsec,getKb());				//stesso nome, per valutazioni istantanee
			x =node.getCoordX();
			y =node.getCoordY();
			d = evalNode(r,node);
			r.add(ind, 1.0);
			e = evaluate(tc,d);  ///
			c.add(ind, e);		///
			outStr += " name: " + name + 
			", x: " + formatter.format(x) + ", y: "+ formatter.format(y)+ "\n";
			if(!name.equals("Target")){
				outStr += "        distance   :   " + formatter.format(d) +"\n"
				+ "        distance."+tc.getName()+" :   " + formatter.format(e) +"\n";

				evalStr += " " +name + ":  "+ formatter.format(e) ;
				if(tc2!=null 
						&& !tc2.getName().equals(tc.getName())
				){
					e2 = evaluate(tc2,d);
					c2.add(ind, e2);	
					outStr +=  "        distance."+tc2.getName()+" :   " + formatter.format(e2) +"\n";
					evalStr += ",  "+ formatter.format(e2);
				}
				evalStr += "\n";
			}
		}
		if (qry!=null){
			if (qryT.isIsRoleQuery()){
				String s =((RoleQuery)qry).evaluateString();        	
//				outStr += "Query evaluation: " + qeval;
				evalStr += "Query evaluation: \n" + s;
			}else{			//OpQuery
				String s = ((OpQuery)qry).evaluateString(true, this.isFuzzy());  //incremental evaluation
//				outStr += "Query evaluation: \n" + s;
				evalStr += "Query evaluation: \n" + s;
			}
		}
		outStr +="\n";        
		outLog(evalStr);
		setStateText(outStr);
		printEvaluation();
	}

	/**
	 * Evaluates membership value for a given value in a given Trapezoid as concrete concept
	 * @param t the Trapezoid
	 * @param d the value
	 * @return the membership value
	 */
	private static double evaluate(Trapezoid t, double d){
		return t.qvalue(d);		
	}
	/**
	 * Evaluates a given Node in a given Role. Used by Distance role, where nodes are moving agents
	 * @param r the Role
	 * @param n the Node
	 * @return the evaluation
	 */
	private static double evalNode (Role r, NodeType n){
		if (r instanceof Distance){
			double xtarg = agents.get(0).getCoordX();
//			System.out.println("pray x: "+ xtarg);
			double ytarg = agents.get(0).getCoordY();
//			System.out.println("pray y: "+ ytarg);
			return ((Distance)r).getValue(xtarg,n.getCoordX(),ytarg,n.getCoordY());
		}
		else return 0.0;
	}
	/**
	 * Writes the given String in a log file.
	 * @param s the String
	 */
	private static void outLog(String s) {
		try {
			sw.write(s + "************************************\n");
		} catch (IOException e) {			
			e.printStackTrace();
		}		
	}
	/**
	 * Creates the ULineEvents in the KnowledgeBase, one each agent
	 */
	public synchronized void createULines(){
		ULineEvent ul;
//		for (NodeType node : agents){  
		for (int i =1; i < agents.size();i++){	//escluso pray
			NodeType node = agents.get(i);
			String name =node.getName();
			ul = new ULineEvent(name,getKb());
			if(!qryT.isIsRoleQuery())
				((OpQuery)qry).getLastvalues().put(name, 0.0);
		}
	}
	/**
	 * Sets the state text 
	 * @param s the text
	 */
	public synchronized static void setStateText(String s){
		monitorFrame.setText2(s);
	}

	/**
	 * Constructs a new Monitor
	 * @param args the arguments
	 * @throws JAXBException 
	 * @throws FileNotFoundException 
	 */
	public static void main(String[] args) throws FileNotFoundException, JAXBException {

		Monitor monitor = new Monitor();		
	}

	/* (non-Javadoc)
	 * @see java.lang.Runnable#run()
	 */
	/**
	 * Runs the current thread substanding this Monitor. Periodically loads current state and
	 * translates it in Abox. At first running creates ULineEvents and shows MTBox.
	 */
	public void run() {		
		Thread me = Thread.currentThread();
		while (thread==me){
			try {
				loadCurrState(stateFile);
//				loadCurrState(currentStateUrl);
			} catch (FileNotFoundException e) {				
				e.printStackTrace();
			} catch (JAXBException e) {				
				e.printStackTrace();
			}
			if (!ulCreated){
				createULines();
				ulCreated=!ulCreated;
				printMTBox();
			}			
			setState();

			try {
				Thread.sleep(500);
			} catch (InterruptedException e) {				
				e.printStackTrace();
			}
		}		
	}
	/**
	 * Prints the evaluation text
	 */
	private static void printEvaluation() {
		monitorFrame.setEvalText(evalStr);

	}



	/**
	 * Sets the output file and starts the new Thread
	 */
	public void start(){

		try {
			sw = new OutputStreamWriter(new FileOutputStream(outFile));

		} catch (FileNotFoundException e) {					
			e.printStackTrace();
		}		
		thread = new Thread(this);
		thread.start();
	}
	/**
	 * Stops current Thread
	 */
	public void stop(){
		try {
			sw.flush();
			sw.close();
		} catch (IOException e) {			
			e.printStackTrace();
		}
		thread = null;
	}



	/**
	 * Gets the KnowledgeBase
	 * @return the kb
	 */
	public static KnowledgeBase getKb() {
		return kb;
	}



	/**
	 * Sets the KnowledgeBase
	 * @param kb the kb to set
	 */
	public static void setKb(KnowledgeBase kb) {
		Monitor.kb = kb;
	}



	/**
	 * Gets the state file
	 * @return the stateFile
	 */
	public File getStateFile() {
		return stateFile;
	}



	/**
	 * Sets the state file
	 * @param stateFile the stateFile to set
	 */
	public void setStateFile(File stateFile) {
		this.stateFile = stateFile;
	}
	/**
	 * @return the subs
	 */
	public boolean isSubs() {
		return subs;
	}
	/**
	 * @param subs the subs to set
	 */
	public void setSubs(boolean subs) {
		this.subs = subs;
	}

	/**
	 * @return the fuzzy
	 */
	public boolean isFuzzy() {
		return fuzzy;
	}

	/**
	 * @param fuzzy the fuzzy to set
	 */
	public void setFuzzy(boolean fuzzy) {
		this.fuzzy = fuzzy;
	}

}
