/**
 * Temporal Quantified fuzzy DL's
 */
package reasoner;

import java.util.ListIterator;
import java.util.TreeSet;
import java.util.Vector;

//import java.util.ListIterator;
//import java.util.TreeSet;
//import java.util.Vector;

/**
 * This class represents a Concrete Concept, having a
 * Trapezoid as membership function and contained in
 * a KnowledgeBase
 * @author marco furlan
 */
public class ConcreteConcept extends Trapezoid {
	/**
	 * the name
	 */
	private String name = "ConcreteConcept";
//	private NumberFormat formatter; 
//	private static double v1, v2;
	/**
	 * the KnowledgeBase
	 */
	protected KnowledgeBase kb;
	/**
	 * the SELF constant
	 */
	private double SELF;
	/**
	 * the NOW constant
	 */
	private double NOW;
	
	/**
	 * Constructor by vertices, max and KnowledgeBase
	 * @param a1 first vertice
	 * @param b1 second vertice
	 * @param c1 third vertice
	 * @param d1 forth vertice
	 * @param max the max value
	 * @param kb the KnowledgeBase
	 */
	public ConcreteConcept(Double a1,Double b1,Double c1,Double d1, Double max, KnowledgeBase kb) {		
		setVertices(a1, b1, c1, d1, max);
		setName("ConcreteConcept");
		setMeaning();
		setKb(kb);
		getKb().addConcreteConcept(this);
	}

	/**
	 * Constructor by name, vertices, max and KnowledgeBase
	 * @param name the name
	 * @param a1 first vertice
	 * @param b1 second vertice
	 * @param c1 third vertice
	 * @param d1 forth vertice
	 * @param max the max value
	 * @param kb the KnowledgeBase
	 */
	public ConcreteConcept(String name,Double a1,Double b1,Double c1,Double d1, Double max, KnowledgeBase kb) {		
		setVertices(a1, b1, c1, d1, max);
		setName(name);
		setMeaning();
		setKb(kb);
		getKb().addConcreteConcept(this);
	}
	
	/**
	 * Constructor by name and KnowledgeBase
	 * @param name the name
	 * @param kb the KnowledgeBase
	 */
	public ConcreteConcept(String name, KnowledgeBase kb) {
		super();		
		setKb(kb);
		setName(name);
		getKb().addConcreteConcept(this);
	}
	
	/**
	 * Converts this ConcreteConcept in a Concept
	 * @return a Concept with same name and kb, empty if it doesn't 
	 * already exists in the kb
	 */
	public Concept toConcept(){
		if (getKb().getConcepts().containsKey(getName()))
			return (Concept)getKb().getConcepts().get(getName());
		else
			return new Concept(getName(),getKb());
	}

	/**
	 * @param name1 the name
	 */
	public	void setName(String name1) {
		name = name1;		
	}

	/** 
	 * @return the name
	 */
	public String getName() {
		if (name.equals("ConcreteConcept"))setName(getMeaning());
		return name;		
	}
	/**
	 * Sets vertices and max value. 
	 * @param a1 first vertice
	 * @param b1 second vertice
	 * @param c1 third vertice
	 * @param d1 forth vertice
	 * @param max the max value
	 */
	protected void setVertices(Double a1,Double b1,Double c1,Double d1, Double max){
		super.setVertices(a1, b1, c1, d1);
		setMaxval(max);
		fixVertices();
	}

	/**
	 * Sets vertices, max and min value. 
	 * @param a1 first vertice
	 * @param b1 second vertice
	 * @param c1 third vertice
	 * @param d1 forth vertice
	 * @param max the max value
	 * @param min the min value
	 */
	protected void setVerticesAndMin(Double a1,Double b1,Double c1,Double d1, Double max, Double min){
		super.setVertices(a1, b1, c1, d1);
		setMaxval(max);
		setMinval(min);
		fixVertices();
	}

//	/**
//	 * @param d the vertice position
//	 * @return the vertice value
//	 */
//	private Double getVertValue(Double d){
//		if (d.equals(geta())||d.equals(getd()))
//			return 0.0;
//		else return 1.0;
//		
//	}
	/**
	 * String representation for this Concrete Concept
	 * @return the String representation
	 * @see reasoner.Trapezoid#toString()
	 */
	public String toString(){
		String s = "";
		s += "ConcreteConcept: " + this.getName() + ": ";
		s += "   " + this.getMeaning() + ": ";
		s += "   0.0/"+formatter.format(geta())+ " + 1.0/"+formatter.format(getb())+
		" + 1.0/"+formatter.format(getc())+ " + 0.0/"+formatter.format(getd());
		s += "\n";
		return s;
	}
	/**
	 * Prints a String representation for this Concrete Concept
	 * @see reasoner.Trapezoid#print()
	 */
	public void print() {		
		System.out.println("ConcreteConcept name: "+ getName());
		System.out.println("ConcreteConcept meaning: "+ getMeaning());
		System.out.println("ConcreteConcept vertices:");
			System.out.printf("%.2f --> 0.0;  ",geta());
			System.out.printf("%.2f --> 1.0;  ",getb());
			System.out.printf("%.2f --> 1.0;  ",getc());
			System.out.printf("%.2f --> 0.0;  ",getd());
			System.out.printf("minVal: %.2f ;  ",getMinval());
			System.out.printf("maxVal: %.2f ;  ",getMaxval());
		System.out.println("");
	}
	
	
	/**
	 * FuzzySet sum between this ConcreteConcept and a second one
	 * @param t the second ConcreteConcept
	 * @return the sum
	 */
	public ConcreteConcept sum(ConcreteConcept t){
		setName("ConcreteConcept");		
		super.sum(t);
		fixVertices();
		return this;
	}

	/**
	* FuzzySet difference between this ConcreteConcept and a second one
	 * @param t the second ConcreteConcept
	 * @return the difference
	 */
	public ConcreteConcept diff(ConcreteConcept t){
		setName("ConcreteConcept");		
		super.diff(t);
		fixVertices();
		return this;
	}
		
	/**
	* FuzzySet product between this ConcreteConcept and a second one
	 * @param t the second ConcreteConcept
	 * @return the product
	 */
	public ConcreteConcept prod(ConcreteConcept t){
		setName("ConcreteConcept");		
		super.prod(t);
		if (t.getMaxval()>1.0)
			setMaxval(Double.MAX_VALUE); 
		fixVertices();
		return this;
	}

	/**
	 * FuzzySet ratio between this ConcreteConcept and a second one.
	 * Possible for divisor non containing zero only!
	 * @param t the second ConcreteConcept
	 * @return the ratio
	 */
	public ConcreteConcept ratio(ConcreteConcept t){
		setName("ConcreteConcept");		
		super.ratio(t);
		if (t.getMaxval()>1.0)
			setMaxval(1.0); 
		else if (t.getMaxval()<=1.0); //
		fixVertices();
		return this;
	}
//	/**
//	 * FuzzySet intersection between this ConcreteConcept and a generic 
//	 * FuzzySet 
//	 * @param f the FuzzySet
//	 * @return the intersection		
//	 */
//	public ConcreteConcept intersection(FuzzySet f){
//		//to do
//		return this;
//	};
//	/**
//	 * FuzzySet union between this ConcreteConcept and a generic 
//	 * FuzzySet 
//	 * @param f the FuzzySet
//	 * @return the union	
//	 */
//	public ConcreteConcept union(FuzzySet f){
//		//to do
//		return this;
//	};
	
	/**
	 * Prints Lambda, the alpha level set for membership values, as a TreeSet<Double>
	 * @param lambda2 the Lambda 
	 */
	public void print(TreeSet<Double> lambda2) {
		Vector<Double> v = new Vector<Double>(lambda2);	
		ListIterator<Double> iter=  v.listIterator(v.indexOf(v.lastElement())+1);
		Object alfa;	
		while(iter.hasPrevious()){
			 alfa =iter.previous();		
			System.out.println("alfa :"+ alfa);
		}
		
	}
	


//	/**	
//	 * @param c
//	 * @return
//	 */
//	public double eval(FuzzySet c){
//		TreeSet<Double> lambda=c.getLambda();
//		System.out.println("lambda: ");
//		print(lambda);
//		double sigma=0.0;
//		Vector<Double> v = new Vector<Double>(lambda);	
//		ListIterator<Double> iter=  v.listIterator(v.indexOf(v.lastElement())+1);
//		Double alfa1, alfa2, qvalue, diff;	
//		Integer card_c_alfa;
//		while(iter.hasPrevious()){
//			 alfa1 =(Double) iter.previous();
//			 card_c_alfa= c.card(alfa1);
//			 System.out.println("card di r_c (con alfa= "+ alfa1+"): "+card_c_alfa);
//			 qvalue=qvalue(card_c_alfa.doubleValue());
//			 System.out.printf("qvalue : %.2f \n", qvalue);
//			 if (iter.hasPrevious()){
//				 alfa2=(Double) iter.previous();
//				 iter.next();//per tornare all'alfa corrente
//			 }else{
//				 alfa2=0.0;
//			 }
//			 System.out.printf("alfa1 : %.2f \n", alfa1);
//			 System.out.printf("alfa2 : %.2f \n", alfa2);
//			 diff=alfa1-alfa2;
//			 System.out.printf("diff : %.2f \n", diff);
//			 sigma += diff*qvalue;
//			 System.out.printf("sigma : %.2f \n", sigma);
//		}
//		
//		return sigma;
//	}
	
//	public Integer card(Double alfa1) {
//		// TODO Auto-generated method stub
//		return null;
//	}
//	public TreeSet<Double> getLambda() {
//		// TODO Auto-generated method stub
//		return null;
//	}
	/**
	 * @return the kb
	 */
	public KnowledgeBase getKb() {
		return kb;
	}
	/**
	 * @param kb the kb to set
	 */
	public void setKb(KnowledgeBase kb) {
		this.kb = kb;
		setNOW(getKb().getNOW());
		setSELF(getKb().getSELF());
	}
	/**
	 * @return the maxval
	 */
	public double getMaxval() {
		return maxval;
	}
	/**
	 * @param maxval the maxval to set
	 */
	public void setMaxval(double maxval) {
		this.maxval = maxval;
	}
	/**
	 * @return the minval
	 */
	public double getMinval() {
		return minval;
	}
	/**
	 * @param minval the minval to set
	 */
	public void setMinval(double minval) {
		this.minval = minval;
	}
	/**
	 * @return the NOW
	 */
	public double getNOW() {
		return NOW;
	}
	/**
	 * @param now the NOW to set
	 */
	public void setNOW(double now) {
		NOW = now;
	}
	/**
	 * @return the SELF
	 */
	public double getSELF() {
		return SELF;
	}
	/**
	 * @param self the SELF to set
	 */
	public void setSELF(double self) {
		SELF = self;
	}
}

