/**
 * Temporal Quantified fuzzy DL's
 */
package reasoner;

/**
 * This abstract class represents the Dynamic Individuals.
 * Concrete subclass are ULineInd and EventInd only. Every
 * DynamicInd belongs necessarly to one and only one ULineEvent
 * @author marco furlan
 */
public abstract class DynamicInd extends Individual {
	protected ULineEvent uLineEvent;

//	/**
//	 * Constructs a new DynamicInd, by name and KnowledgeBase
//	 * @param name the name
//	 * @param kb the KnowledgeBase
//	 */
//	public DynamicInd(String name, KnowledgeBase kb) {
//		super(name, kb);		
//	}

	/**
	 * Constructs a new DynamicInd, by name, ULineEvent, time and KnowledgeBase
	 * @param name the name	 
	 * @param line the ULineEvent
	 * @param time the time
	 * @param kb the KnowledgeBase
	 */
	public DynamicInd(String name, ULineEvent line, double time,
			KnowledgeBase kb) {
		super(name, kb);
		setTime(time);
//		getKb().addIndividual(this);
		setULineEvent(line);		
		line.add(time, this);	
	}
	
	/**
	 * Evaluates the membership value for this DynamicInd in the given
	 * Concept
	 * @param c the Concept
	 * @see reasoner.Individual#degree(reasoner.Concept)
	 */
	public abstract double degree(Concept c) ;

	/**
	 * @return the ULineEvent
	 */
	public ULineEvent getULineEvent() {
		return uLineEvent;
	}

	/**
	 * @param line the ULineEvent to set
	 */
	public void setULineEvent(ULineEvent line) {
		uLineEvent = line;	
	}
}
