/**
 * Temporal Quantified fuzzy DL's
 */
package reasoner;

import java.text.NumberFormat;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;

/**
 * This class represents a Query in the form (Qt)(Q)AopB, where A and B are 
 * ConcreteConcept, op is binary or unary operator between Concepts, Qt is
 * temporal Quantifier and Q is Quantifier.
 * @author marco furlan
 */
public class OpQuery implements Query {
	/**
	 * The first ConcreteConcept
	 */
	private ConcreteConcept tc1;
	/**
	 * The second ConcreteConcept
	 */
	private ConcreteConcept  tc2;
	/**
	 * The first Concept
	 */
	private Concept c1; 
	/**
	 * The second Concept
	 */
	private Concept c2;
	/**
	 * The Quantifier
	 */
	private Quantifier q;
	/**
	 * The Temp Quantifier
	 */
	private Quantifier tempq;
	
	/**
	 * The operator
	 */
	private String operator;
	/**
	 * The KnowledgeBase
	 */
	private KnowledgeBase kb;
	/**
	 * The last value map, for incremental evaluation
	 */
	private Map<String, Double> lastvalues;
	
	private Role is;
	private Concept d;
	private NumberFormat formatter; 

	
	/**
	 * Constructor by two ConcreteConcepts, binary operator and  KnowledgeBase
	 * @param tc1 the first ConcreteConcept
	 * @param operator the operator
	 * @param tc2 the second ConcreteConcept
	 * @param kb the KnowledgeBase
	 */
	public OpQuery(ConcreteConcept tc1, String operator,ConcreteConcept tc2, KnowledgeBase kb) {
		super();
		setTc1(tc1);
		setTc2(tc2);
		setC1(trap2Concept(tc1));
		setC2(trap2Concept(tc2));
		setOperator(operator);
		setKb(kb);
		getKb().setQuery(this);
		setLastvalues(new TreeMap<String, Double>());
		formatter = NumberFormat.getNumberInstance();
		formatter.setMaximumFractionDigits(6);
	}

	/** 
	 * Constructor by Quantifier, two ConcreteConcepts, binary operator and  KnowledgeBase
	 * @param q the Quantifier
	 * @param tc1 the first ConcreteConcept
	 * @param operator the operator
	 * @param tc2 the second ConcreteConcept
	 * @param kb the KnowledgeBase
	 */
	public OpQuery(Quantifier q, ConcreteConcept tc1, String operator,ConcreteConcept tc2, KnowledgeBase kb) {
		this(tc1,operator,tc2,kb);
		setQ(q);
	}
	
	/** 
	 * Constructor by Quantifier, two ConcreteConcepts, binary operator and  KnowledgeBase
	 * @param tq the temporal Quantifier
	 * @param q the Quantifier
	 * @param tc1 the first ConcreteConcept
	 * @param operator the operator
	 * @param tc2 the second ConcreteConcept
	 * @param kb the KnowledgeBase
	 */
	public OpQuery(Quantifier tq, Quantifier q, ConcreteConcept tc1, String operator,ConcreteConcept tc2, KnowledgeBase kb) {
		this(tc1,operator,tc2,kb);
		setQ(q);
		this.setTempq(tq);
	}

	/**
	 * Converts a ConcreteConcept in a Concept
	 * @param tc the ConcreteConcept
	 * @return the Concept
	 */
	private Concept trap2Concept(ConcreteConcept tc) {		
		return tc.toConcept();
	}


	/* (non-Javadoc)
	 * @see reasoner.Query#evaluate()
	 */
	public double evaluate() {
		String s = "";		
		return 0;
	}
	
	/**
	 * Evaluates this OpQuery and returns a String result. 
	 * @param incremental if <code>true</code> then incremental evaluation
	 * @param fuzzy if <code>true</code> then fuzzy evaluation for subsumption
	 * @return the result
	 */
	public synchronized String evaluateString(boolean incremental, boolean fuzzy) {
		HashMap<String,ULineEvent> uLines;
		String name1 = this.getC1().getName(), 
		name2 = this.getC2().getName();
		Set<String> ulNames;
		Iterator<String> it ;
		String op = this.getOperator();
		String s = "", s1="";
		ULineEvent ul;
		Individual xlast,xo ;
		Concept newConc = null;
		is = new Role ("is", getKb());
		String n = "(" + name1 + ")"+op+"(" + name2 + ")";
//		d = new Concept(n, this.getKb());
		double lastVal, newVal, qval;
		if(op.equals("Until")){
			if (incremental){
				newConc = new Concept(n, this.getKb());
				s += "      ("+ name1 +")"+op+"("+ name2 + "): {";
				uLines = this.getKb().getULineEvents();
				ulNames = uLines.keySet();			
				it = ulNames.iterator();
				while(it.hasNext()){
					String name = (String)it.next();
					ul = (ULineEvent) uLines.get(name);
					xlast = ul.getLast();
					xo = ul.getMyULine();				
					lastVal = (getLastvalues().get(name)!=null?getLastvalues().get(name):0.0);
					newVal =this.getC1().until(this.getC2(), xlast, lastVal);
					getLastvalues().remove(name);
					getLastvalues().put(name, newVal);
//					is.add(xo, 1.0);
					newConc.add(xo, newVal);		//sostituisce il valore eventuale precedente
					if (newVal > 0.0)
						s += " <"+name+", "	+formatter.format(newVal)+">, ";	
				}
				if (s.contains(","))
					s = s.substring(0, s.lastIndexOf(","));
				s += "}\n" + s1 ;
			}else {
				newConc = this.getC1().until(this.getC2());
				s += newConc.toNormalStringUl() + s1;
			}

		}else if(op.equals("Since")){
			newConc = this.getC1().since(this.getC2());
			s += newConc.toNormalStringUl() + s1;
		}else if(op.equals("Intersection")){
			Concept ints = this.getC1().intersection(this.getC2());
			s += ints.toString() + s1;
			if (null!= this.getTempq())
				s += ints.tempQ(this.getTempq()).toStringUl();
		}else if(op.equals("Union")){
			Concept union = this.getC1().union(this.getC2());
			s += union.toString() + s1;
			if (null!= this.getTempq())
				s += union.tempQ(this.getTempq()).toStringUl();			
		}else if(op.equals("SubsNext")||op.equals("Incr")||
				op.equals("Decr")||op.equals("Const")||
				op.equals("SubsPrev")){
			this.getC1().createTendencyConcepts(fuzzy);
			s += getKb().getConcepts().get("SubsNext(" + name1 + ")").toNormalStringUl();
			s += getKb().getConcepts().get("SubsPrev(" + name1 + ")").toNormalStringUl();
			s += getKb().getConcepts().get("Incr(" + name1 + ")").toNormalStringUl();
			s += getKb().getConcepts().get("Decr(" + name1 + ")").toNormalStringUl();
			s += getKb().getConcepts().get("Const(" + name1 + ")").toNormalStringUl();
			s += this.getC1().linearInterpolation2String();
			newConc = getKb().getConcepts().get(op + "(" + name1 + ")");
			
		}

		if (null!=this.getQ() && !op.equals("Union") && !op.equals("Intersection")){
//			if (!(op.equals("Until") && incremental)){
			if (null != newConc){
				uLines = this.getKb().getULineEvents();
				ulNames = uLines.keySet();			
				it = ulNames.iterator();
//				while(it.hasNext()){
				for (String ulname : ulNames){
					ul = uLines.get(ulname);
					xo = ul.getMyULine();
					is.add(xo, 1.0);
				}
				

//				if (this.getQ().isRelative()){
				qval = this.getQ().eval(is, newConc);
			}else
				qval = 0.0;		
			s += "\n      "+ this.getMeaning()+": "+
			formatter.format(qval)+"\n";	
		}
		return s;
	}

	
	
	/**
	 * @return the operator
	 */
	public String getOperator() {
		return operator;
	}


	/**
	 * @param operator the operator to set
	 */
	public void setOperator(String operator) {
		if(operator.trim().equalsIgnoreCase("until"))
			this.operator = "Until";
		else if(operator.trim().equalsIgnoreCase("subsumedNext")||
				operator.trim().equalsIgnoreCase("SubsNext"))
			this.operator = "SubsNext";
		else if(operator.trim().equalsIgnoreCase("subsumedPrev")||
				operator.trim().equalsIgnoreCase("SubsPrev"))
			this.operator = "SubsPrev";
		else if (operator.trim().equalsIgnoreCase("Incr"))
			this.operator = "Incr";
		else if (operator.trim().equalsIgnoreCase("Decr"))
			this.operator = "Decr";
		else if (operator.trim().equalsIgnoreCase("Incr"))
			this.operator = "Incr";
		else if (operator.trim().equalsIgnoreCase("Const"))
			this.operator = "Const";
		else this.operator = operator;
	}


	/**
	 * @return the tc1
	 */
	public ConcreteConcept getTc1() {
		return tc1;
	}


	/**
	 * @param tc1 the tc1 to set
	 */
	public void setTc1(ConcreteConcept tc1) {
		this.tc1 = tc1;
	}


	/**
	 * @return the tc2
	 */
	public ConcreteConcept getTc2() {
		return tc2;
	}


	/**
	 * @param tc2 the tc2 to set
	 */
	public void setTc2(ConcreteConcept tc2) {
		this.tc2 = tc2;
	}
	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString(){
		String s = "Query:  " + getMeaning() + "\n";
		return s;
	}

	/* (non-Javadoc)
	 * @see reasoner.Query#getMeaning()
	 */
	public String getMeaning(){
		String op = this.getOperator();
		String s = "";
		if(getTempq()!=null)
			s += "(Temp" + getTempq().getName() + ")";
		if(getQ()!=null)
			s += "(" + getQ().getName() + ")";
		if(op.equals("SubsNext")||op.equals("Incr")||
				op.equals("Decr")||op.equals("Const")||
				op.equals("SubsPrev")){
			s += op+"("+ getTc1().getName() + ")";
		}else
		if(getTc1()!=null && op!=null && getTc2()!=null){
			s += "(" + getTc1().getName() + ")";	
			
			s += op + "(" + getTc2().getName() + ")";
		}
		return s;	

	}

	/**
	 * @return the kb
	 */
	public KnowledgeBase getKb() {
		return kb;
	}


	/**
	 * @param kb the kb to set
	 */
	public void setKb(KnowledgeBase kb) {
		this.kb = kb;
	}


	/**
	 * @return the c1
	 */
	public Concept getC1() {
		return c1;
	}


	/**
	 * @param c1 the c1 to set
	 */
	public void setC1(Concept c1) {
		this.c1 = c1;
	}


	/**
	 * @return the c2
	 */
	public Concept getC2() {
		return c2;
	}


	/**
	 * @param c2 the c2 to set
	 */
	public void setC2(Concept c2) {
		this.c2 = c2;
	}


	/**
	 * @return the lastvalues
	 */
	public Map<String, Double>  getLastvalues() {
		return lastvalues;
	}
	

	/**
	 * @param lastvalues the lastvalues to set
	 */
	public void setLastvalues(Map<String, Double> lastvalues) {
		this.lastvalues = lastvalues;
	}


	/**
	 * @return the q
	 */
	public Quantifier getQ() {
		return q;
	}


	/**
	 * @param q the q to set
	 */
	public void setQ(Quantifier q) {
		this.q = q;
	}

	/**
	 * @return the d
	 */
	public Concept getD() {
		return d;
	}

	/**
	 * @param d the d to set
	 */
	public void setD(Concept d) {
		this.d = d;
	}

	/**
	 * @return the is
	 */
	public Role getIs() {
		return is;
	}

	/**
	 * @param r the r to set
	 */
	public void setIs(Role r) {
		this.is = r;
	}

	/**
	 * @return the tempq
	 */
	public Quantifier getTempq() {
		return tempq;
	}

	/**
	 * @param tempq the tempq to set
	 */
	public void setTempq(Quantifier tempq) {
		this.tempq = tempq;
	}

}
