/**
 * Temporal Quantified fuzzy DL's
 */
package util;

import java.util.ArrayList;

/**
 * An util class for linear interpolation of an ArrayList of Point
 * @author marco furlan
 * 
 */
public class LinearInterpolation {
	/**
	 * The list of Points
	 */
	private ArrayList<Point> points;
	/**
	 * The angular coefficient
	 */
	private Double m;
	/**
	 * The y-axis intercept
	 */
	private Double q;

	/**
	 * The constructor.
	 * @param points the ArrayList of Point to interpolate
	 */
	public LinearInterpolation(ArrayList<Point> points) {
		super();
		this.points = points;
	}
	
	/**
	 * Interpoles the ArrayList of Point, using the
	 * interpolation formula minimizing the sum of the squared
	 * distances from points to rect line
	 * 
	 */
	public void interpole (){
		double a,b,c,d, x, y, n, den;
		a=0; 								//x-coord sum
		b=0; 								//y-coord sum
		c=0; 								//sum of products(x-coord, y-coord)
		d=0;								//sum of (x-coord)^2
		n = getPoints().size();				//point number
		for (Point point:getPoints() ){
			x = point.getX();
			y = point.getY();
			a += x;		
			b += y;		
			c += x* y;
			d += x* x;
		}
		if (n == 1){
			setM(0.0);
			setQ(b);
		}else {
		den = a*a - n*d;					//den = a^2 - n*d
		setM((a*b-n*c )/den);				//m = (a*b - n*c)/den
		setQ((a*c - b*d)/den);				//q = (a*c - b*d)/den
		}
	}

	/**
	 * @return the angular coefficient m
	 */
	public Double getM() {
		return m;
	}

	/**
	 * @param m the the angular coefficient m to set
	 */
	public void setM(Double m) {
		this.m = m;
	}

	/**
	 * @return the y-intersection q
	 */
	public Double getQ() {
		return q;
	}

	/**
	 * @param q the y-intersection q to set
	 */
	public void setQ(Double q) {
		this.q = q;
	}

	/**
	 * @return the points
	 */
	public ArrayList<Point> getPoints() {
		return points;
	}

	/**
	 * @param points the points to set
	 */
	public void setPoints(ArrayList<Point> points) {
		this.points = points;
	}

//	public static void main(String[] args){
//		ArrayList<Point> al = new ArrayList<Point>();
//		Point pnt;		
//		pnt = new Point(1.0, 9.0);
//		al.add(pnt);
//		pnt = new Point(4.0, 2.0);
//		al.add(pnt);
//		pnt = new Point(7.0, 11.0);
//		al.add(pnt);
//		LinearInterpolation li = new LinearInterpolation(al);
//		li.interpole();		
//		System.out.printf("interpolazione lineare dei dati\n m: %.2f; q: %.2f", li.getM(), li.getQ());
//	}
}
