package util;

import java.io.*;

/**
*A FileFilter for the JFileChoosers: accepts text file (.xml) only
*@author Marco Furlan
*/
public class XmlFileFilter extends javax.swing.filechooser.FileFilter {
	public XmlFileFilter() {
		super();
	}

	/**
	*Says whether a given file is allowed.
	*All directories and xml files (.xml) only are allowed 
	*@param f the file
	*@return <code>true</code> if the file is allowed
	*/
	public boolean accept(File f) {
		if(f != null) {
	   		if(f.isDirectory())
				return true;
	   		String extension = getExtension(f);
	   		if(extension != null && extension.equals("xml")) 
				return true;
		}
		return false;
   }

	/**
	*Returns the extension of a given file
	*@param f the file
	*@return the extension
	*/
  public String getExtension(File f) {
		if(f != null) {
			String filename = f.getName();
	   		int i = filename.lastIndexOf('.');
			if(i>0 && i<filename.length()-1) {
				return filename.substring(i+1).toLowerCase();
	   		 }
		}
		return null;
	}
	
	/**
	*Gets the description for the text files
	*@return the description
	*/
	public String getDescription(){
		return"xml files (*.xml)";
	}
}

