/**
 * Temporal Quantified fuzzy DL's
 */
package reasoner;

/**
 * This abstract class represents the generic Individuals. Every Individual
 * has a name and a time and is contained in a KnowledgeBase. Subclass
 * are StaticInd and DynamicInd only.
 * @author marco furlan
 *
 */
public abstract class Individual implements Comparable<Individual> {
	protected String name;	
	protected double time;
	protected  static double SELF;
	protected  static double NOW;
	protected KnowledgeBase kb;
	
	/**
	 * Constructor for the generic Individual, by name and KnowledgeBase	
	 * @param name the name
	 * @param kb the KnowledgeBase
	 */
	public Individual (String name,KnowledgeBase kb){
		setKb(kb);
		setName(name);
		getKb().addIndividual(this);
	};
	
	/**
	 * Evaluates the membership value for this Individual in the given
	 * Concept
	 * @param c the Concept
	 */
	public abstract double degree(Concept c);
	
	/**
	 * @return the name
	 */
	public String getName() {
		return name;
	}

	/**
	 * @param name the name to set
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * If this individual does'nt belong to any UniversalLine, it's
	 * time is NOW; otherwise, it's time is set on creation
	 * @return the time
	 */
	public double getTime() {
//		if (this.getULine()==null)
//			return this.getNOW();
		if (Double.valueOf(this.time) == null)
			return getNOW();
		return time;
	}

	/**
	 * @param time the time to set
	 */
	public void setTime(double time) {
		this.time = time;
	}

//	/**
//	 * @param name
//	 * @param line
//	 * @param time
//	 */
//	public Individual(String name, ULinelEvent line, double time,KnowledgeBase kb) {
//		super();		
//		setKb(kb);
//		setName (name);
//		setTime(time);
//		setULine(line);		
//		line.add(time, this);
//		getKb().addIndividual(this);
//	}


	/**
	 * Compares by name this Individual with a given one
	 * @param i the given Individual
	 * @return the value 0 if this Individual name is lexicographical equal
	 * to the argument Individual name; a value less than 0 if this Individual name
	 * is lexicographically less than the argument Individual name; and a value 
	 * greater than 0 if this Individual name is lexicographically greater than the 
	 * argument Individual name.
	 */
	public int compareTo(Individual i) {		
//		return new Double(this.getTime()).compareTo(((Individual)i).getTime());
		return this.getName().compareTo(i.getName());		
	}

	/**
	 * @return the KnowledgeBase kb
	 */
	public KnowledgeBase getKb() {
		return kb;
	}

	/**
	 * @param kb the KnowledgeBase to set
	 */
	public void setKb(KnowledgeBase kb) {
		this.kb = kb;
		setNOW(getKb().getNOW());
		setSELF(getKb().getSELF());
	}

	/**
	 * @return the constant SELF, set by the KnowledgeBase
	 */
	public double getSELF() {
		return SELF;
	}

	/**
	 * @param self the constant SELF to set
	 */
	private void setSELF(double self) {
		SELF = self;
	}

	/**
	 * @return the constant NOW, set by the KnowledgeBase
	 */
	public double getNOW() {
		return this.getKb().getNOW();
	}

	/**
	 * @param now the constant NOW to set
	 */
	private void setNOW(double now) {
		NOW = now;
	}

}
