/**
 * Temporal Quantified fuzzy DL's
 */
package reasoner;

import java.text.NumberFormat;
import java.util.Iterator;
import java.util.TreeMap;
import java.util.TreeSet;

/**
 * This class represents the Concept of individuals being the 
 * events corresponding to the same ULine; it contains, for sake of simplicity,
 * the corresponding ULine too.
 * @author marco furlan
 */
public class ULineEvent extends Event implements Comparable<Concept>{
//	/**
//	 * the ULineEvent name
//	 */
//	private String name;
	/**
	 * the time map
	 */
	private TreeMap<Double, Individual> timeMap;
	
	/**
	 * Constructor by name and KnowledgeBase
	 * @param name the name
	 * @param kb the KnowledgeBase
	 */
	public ULineEvent(String name,KnowledgeBase kb) {
		super(name, kb);
//		setName(name);
		setTimeMap(new TreeMap<Double, Individual>());
		kb.addULineEv(this);
		setULine();
	}
	
	/**
	 * Gets the maximum value gained by this ULineEvent on
	 * a given Concept
	 * @param a the Concept
	 * @return the maximum value
	 */	
	public double max(Concept a){
		System.out.print(this.getName()+".eval("+a.getName()+").value(): ");
		Individual xm = firstMax(a);
		if (xm!= null) {
			System.out.println(a.get(xm));
			return a.get(xm);}
		else {
			System.out.println(0.0);
			return 0.0;
		}
	}
	/**	 
	 * Gets the first Individual in this ULineEvent, ordered by time, or null if 
	 * this ULineEvent is empty.
	 * @return the first Individual or null
	 */
	public Individual getFirst(){
		if (this.getTimeMap().isEmpty())
			return null;
		double firstTime = this.getTimeMap().firstKey();
		Individual first = this.getTimeMap().get(firstTime);
//		if (firstTime == getSELF())
		if (first instanceof ULineInd)
			return this.getNext(first);
		else return first; 
	}
	/**	 
	 * Gets the last Individual in this ULineEvent, ordered by time, or null if 
	 * this ULineEvent is empty.
	 * @return the last Individual or null
	 */
	public Individual getLast(){
		if (this.getTimeMap().isEmpty())
			return null;
		double lastTime = this.getTimeMap().lastKey();
		Individual last = this.getTimeMap().get(lastTime);
//		if (lastTime == getSELF())
		if (last instanceof ULineInd)
			return this.getPrevious(last);
		else return last; 
	}

	/**
	 * Gets next Individual, after given Individual, in this ULineEvent, ordered by time, or null if 
	 * next Individual does not exist.	 
	 * @param ind the given Individual
	 * @return next Individual
	 */
	public Individual getNext(Individual ind){
		double myTime, nextTime;
		Individual next;
		myTime = ind.getTime();
		try{
			nextTime = (this.getTimeMap()).higherKey(myTime);
		}catch (java.lang.NullPointerException e){
			return null;
		}		
		next = this.getTimeMap().get(nextTime);
//		if (nextTime == getSELF())
		if (next instanceof ULineInd)
			return getNext(next);
		else return next; 
	}
	/**
	 * Returns the concept membership value for the individual belonging
	 * this ULineEvent, corresponding to given time.
	 * @param time the time
	 * @param c the Concept
	 * @return the Concept membership value
	 */
	public double get (double time, Concept c){
		Individual ind = this.getIndividual(time);
		if (null==ind)
			return 0.0;	
		return c.get(ind);	//-->0.0 if c does not contain ind
	}

	/**
	 * Returns previous Individual, in time ordering, before given Individual or null 
	 * if such previous Individual does not exist.
	 * @param ind the given Individual
	 * @return the previous Individual 
	 */
	public Individual getPrevious(Individual ind){
		double myTime, lastTime;
		Individual last;
		myTime = ind.getTime();
		try{		
			lastTime = this.getTimeMap().lowerKey(myTime);
		}catch (java.lang.NullPointerException e){
			return null;
		}	
		last = this.getTimeMap().get(lastTime);
//		if (lastTime == getSELF())
		if (last instanceof ULineInd)
			return getPrevious(last);
		else return last; 
	}
	/**
	 * Returns a new ULineEvent identical to the present one, except for the last Individual 
	 * that is missing
	 * @return the ULineEvent
	 */
	public ULineEvent getWithoutLast(){
		ULineEvent ul = new ULineEvent(this.getName()+"WithoutLast",getKb());
		Individual lastInd = this.getLast();
		double lastTime = lastInd.getTime();
		ul.getTimeMap().putAll(this.getTimeMap());
//		ul.setTimeMap(this.getTimeMap());		
		ul.getTimeMap().remove(lastTime);
//		ul.setMap(this.getMap());
		ul.getEvents().putAll(this.getEvents());
		ul.getEvents().remove(lastInd);
//		ul.setULineMap(this.getULineMap());
		ul.getULineMap().putAll(this.getULineMap());
		return ul;
	}
	/**
	 * Returns the Individual corrisponding on the first maximum value on the Concept a
	 * @param a the Concept
	 * @return the Individual
	 */
	public Individual firstMax(Concept a){
		System.out.print(this.getName()+".firstMax("+a.getName()+"): ");
		double max = 0.0;	
		double time = 0.0;
		Individual xm = null;
		TreeSet<Individual> keys = new TreeSet<Individual>(a.getEvents().keySet());
		Iterator<Individual> iter;		
		iter = keys.iterator();		
		while(iter.hasNext()){
			Individual ind =(Individual)iter.next();			
			if (a.containsKey(ind)&&(((DynamicInd)ind).getULineEvent()==this)){
				Double val = a.get(ind);
				if (val.compareTo(max)>0 ||
						(val.compareTo(max)==0 && ind.getTime()<time)){
					max = val;
					xm = ind;
				}}
		}
		System.out.println(xm.getName());	
		return xm;

	}

	/**
	 * Returns the Individual corrisponding on the last maximum value on the Concept a
	 * @param a the Concept
	 * @return the Individual
	 */
	public Individual lastMax(Concept a){
		System.out.print(this.getName()+".lastMax("+a.getName()+"): ");
		double max = 0.0;
		double time = 0.0;
		Individual xm = null;
		TreeSet<Individual> keys = new TreeSet<Individual>(a.getEvents().keySet());
		Iterator<Individual> iter;		
		iter = keys.iterator();		
		while(iter.hasNext()){
			Individual ind =(DynamicInd)iter.next();			
			if (a.containsKey(ind)&& (((DynamicInd)ind).getULineEvent()==this)){
				Double val = a.get(ind);
				if (val.compareTo(max)>=0||
						(val.compareTo(max)==0 && ind.getTime()>time)){
					max = val;
					xm = ind;
					time= ind.getTime();
				}}
		}
		if(xm!=null)
			System.out.println(xm.getName());	
		else
			System.out.println("null");
		return xm;

	}

	/* (non-Javadoc)
	 * @see reasoner.Concept#print()
	 */
	public  void print() {
//		Set keys =getEvKeys();
		Iterator<Double> iter;
//		iter= keys.iterator();
//		System.out.println("ULineEvent: " + getName());
//		while(iter.hasNext()){
//		Individual ind =(Individual)iter.next();
//		Double time= (Double)ind.getTime();			
//		System.out.printf("%s --> time: %.2f;  ",ind.getName(), time);
//		}
//		System.out.println("");

		TreeSet<Double> t = getTimeSet();
		iter = t.iterator();
		System.out.println("ULineEvent: " + getName());
		while(iter.hasNext()){
			Double d =(Double)iter.next();
			Individual ind = (Individual)getTimeMap().get(d);			
			System.out.printf("%s --> time: %.2f;  ",ind.getName(), d);
		}
		System.out.println("");
	}

	/* (non-Javadoc)
	 * @see reasoner.Concept#toString()
	 */
	public String toString(){		
		String s = "";
		Individual ind;
		Double d;
		NumberFormat formatter = NumberFormat.getInstance();
		formatter.setMaximumFractionDigits(2);	
		TreeSet<Double> t = getTimeSet();
		Iterator<Double> iter= t.iterator();
		s += "ULineEvent: " + getName() + ": ";
		while(iter.hasNext()){
			d= (Double)iter.next();	
			ind =(Individual)getTimeMap().get(d);

			s += ind.getName() + " --> time: " + formatter.format(d) + "; ";		
		}
		s += "\n";

		return s;
	}

	/**
	 * Creates a new ULine for this ULineEvent
	 */
	private void setULine(){
//		System.out.println("setULine SELF: "+ getSELF());
		Individual uLine = new ULineInd(this.getName().toLowerCase()+"0",this,getSELF(),getKb());
	}
	/**
	 * Adds an Individual and it's time in this ULineEvent
	 * @param time the time
	 * @param i the Individual
	 */
	public void add(Double time, Individual i){
		super.add(i, 1.0);
		getTimeMap().put(time, i);
	}
	/**
	 * @return the time set
	 */
	public TreeSet<Double> getTimeSet(){
		return new TreeSet<Double>(getTimeMap().keySet());
	}
//	/**
//	* @param r
//	*/
//	public ULineEvent(TreeMap<Individual, Double> r, TreeMap<Double, Individual> time) {
//	super(r);
	//
//	}
//	/**
//	 * @return the name
//	 */
//	public String getName() {
//		return name;
//	}
//	/**
//	 * @param name the name to set
//	 */
//	public void setName(String name) {
//		this.name = name;
//	}
	/**
	 * @return the timeMap
	 */
	public TreeMap<Double, Individual> getTimeMap() {
		return timeMap;
	}
	/**
	 * @param timeMap the timeMap to set
	 */
	public void setTimeMap(TreeMap<Double, Individual> timeMap) {
		this.timeMap = timeMap;
	}

	/**
	 * Gets Individual with a given time
	 * @param time the time
	 * @return the Individual
	 */
	public Individual getIndividual(Double time) {	
		if (this.getTimeMap().containsKey(time))
			return getTimeMap().get(time);
		else return null;
	}
	/* (non-Javadoc)
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 */
	public int compareTo(Concept i) {		
		return this.getName().compareTo(((ULineEvent)i).getName());
	}
	/**
	 * Returns ULineInd corresponding to this ULineEvent
	 * @return the ULineInd
	 */
	public ULineInd getMyULine(){
		return (ULineInd)getIndividual(getSELF());
	}

}
